"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertBigIntToString = convertBigIntToString;
exports.getSmartAccountAddressFromInitialSigner = getSmartAccountAddressFromInitialSigner;
exports.isSmartAccountDeployed = isSmartAccountDeployed;
exports.getInitializerCalldata = getInitializerCalldata;
const viem_1 = require("viem");
const AccountFactory_js_1 = require("./abis/AccountFactory.js");
const constants_js_1 = require("./constants.js");
function convertBigIntToString(value) {
    if (typeof value === 'bigint') {
        return value.toString();
    }
    else if (Array.isArray(value)) {
        return value.map(convertBigIntToString);
    }
    else if (typeof value === 'object' && value !== null) {
        return Object.fromEntries(Object.entries(value).map(([key, val]) => [
            key,
            convertBigIntToString(val),
        ]));
    }
    return value;
}
async function getSmartAccountAddressFromInitialSigner(initialSigner, publicClient) {
    const addressBytes = (0, viem_1.toBytes)(initialSigner);
    const salt = (0, viem_1.keccak256)(addressBytes);
    const accountAddress = (await publicClient.readContract({
        address: constants_js_1.SMART_ACCOUNT_FACTORY_ADDRESS,
        abi: AccountFactory_js_1.default,
        functionName: 'getAddressForSalt',
        args: [salt],
    }));
    return accountAddress;
}
async function isSmartAccountDeployed(publicClient, address) {
    try {
        const bytecode = await publicClient.getCode({
            address: address,
        });
        return bytecode !== null && bytecode !== '0x' && bytecode !== undefined;
    }
    catch (error) {
        console.error('Error checking address:', error);
        return false;
    }
}
function getInitializerCalldata(initialOwnerAddress, validatorAddress, initialCall) {
    return (0, viem_1.encodeFunctionData)({
        abi: [
            {
                name: 'initialize',
                type: 'function',
                inputs: [
                    { name: 'initialK1Owner', type: 'address' },
                    { name: 'initialK1Validator', type: 'address' },
                    { name: 'modules', type: 'bytes[]' },
                    {
                        name: 'initCall',
                        type: 'tuple',
                        components: [
                            { name: 'target', type: 'address' },
                            { name: 'allowFailure', type: 'bool' },
                            { name: 'value', type: 'uint256' },
                            { name: 'callData', type: 'bytes' },
                        ],
                    },
                ],
                outputs: [],
                stateMutability: 'nonpayable',
            },
        ],
        functionName: 'initialize',
        args: [initialOwnerAddress, validatorAddress, [], initialCall],
    });
}
//# sourceMappingURL=utils.js.map